@@ -24,6 +24,7 @@
 #include <goffice/graph/gog-axis-line-impl.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-style.h>
@@ -774,11 +775,8 @@ gog_tool_select_axis_point (GogView *view, double x, double y, GogObject **gobj)
 static void
 gog_tool_select_axis_render (GogView *view)
 {
-	if (gog_tool_bound_is_valid_axis (view)) {
-		ArtVpath *path = gog_renderer_get_rectangle_vpath (&view->allocation);
-		gog_renderer_draw_sharp_path (view->renderer, path);
-		art_free (path);
-	}
+	if (gog_tool_bound_is_valid_axis (view))
+		gog_renderer_draw_selection_rectangle (view->renderer, &view->allocation);
 }
 
 static GogTool gog_axis_tool_select_axis = {
@@ -949,12 +947,15 @@ axis_line_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 	GogStyle *style = axis_base->base.style;
 	GOGeometryAABR txt_aabr;
 	GOGeometryOBR txt_obr;
+	GOGeometryOBR *obrs = NULL;
+	GOGeometrySide label_anchor = GO_SIDE_AUTO;
 	double line_width;
 	double axis_length, axis_angle, label_padding;
 	double cos_alpha, sin_alpha;
 	double pos;
 	double minor_tick_len, major_tick_len, tick_len;
 	double padding = gog_axis_base_get_padding (axis_base);
+	double label_size_max = 0;
 	unsigned i, tick_nbr;
 	gboolean is_line_visible;
 
@@ -972,7 +973,7 @@ axis_line_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 	gog_renderer_get_text_OBR (renderer, "0", TRUE, &txt_obr);
 	label_padding = txt_obr.w;
 
-	total_bbox.x = x; total_bbox.y = y; 
+	total_bbox.x = x; total_bbox.y = y;
 	total_bbox.w = w; total_bbox.h = h;
 	bbox.x = x -= gog_renderer_pt2r_x (renderer, padding * cos_alpha);
 	bbox.y = y -= gog_renderer_pt2r_y (renderer, padding * sin_alpha);
@@ -1006,30 +1007,46 @@ axis_line_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 
 	tick_nbr = gog_axis_get_ticks (axis_base->axis, &ticks);
 
-	if (!draw_labels)
+	if (!draw_labels || tick_nbr < 1)
 		return total_bbox;
 
 	map = gog_axis_map_new (axis_base->axis, 0., axis_length);
 
+	obrs = g_new0 (GOGeometryOBR, tick_nbr);
 	for (i = 0; i < tick_nbr; i++) {
 		if (ticks[i].label != NULL) {
+			GOGeometryOBR *obr = obrs + i;
+			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, obr);
+			if (obr->w > label_size_max
+			    || obr->h > label_size_max) {
+				label_size_max = MAX (obr->w, obr->h);
+				label_anchor = go_geometry_calc_label_anchor (obr, axis_angle);
+			}
+		}
+	}
+
+	for (i = 0; i < tick_nbr; i++) {
+		if (ticks[i].label != NULL) {
+			GOGeometryOBR *obr = obrs + i;
 			pos = gog_axis_map_to_view (map, ticks[i].position);
-			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, &txt_obr);
-			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, axis_angle, tick_len, side);
-			txt_obr.x += x + pos * cos (axis_angle);
-			txt_obr.y += y + pos * sin (axis_angle);
-			go_geometry_OBR_to_AABR (&txt_obr, &txt_aabr);
+			obr->w += label_padding;
+			go_geometry_calc_label_position (obr, axis_angle, tick_len,
+							 side, label_anchor);
+			obr->x += x + pos * cos (axis_angle);
+			obr->y += y + pos * sin (axis_angle);
+			go_geometry_OBR_to_AABR (obr, &txt_aabr);
 			go_geometry_AABR_add (&total_bbox, &txt_aabr);
 		}
 	}
+	g_free (obrs);
+
 	gog_axis_map_free (map);
 
 	return total_bbox;
 }
 
 static void
-axis_line_render (GogAxisBase *axis_base, 
+axis_line_render (GogAxisBase *axis_base,
 		  GogAxisBaseView *axis_base_view,
 		  GogRenderer *renderer,
 		  double x, double y, double w, double h,
@@ -1043,7 +1060,8 @@ axis_line_render (GogAxisBase *axis_base,
 	GogStyle *style = axis_base->base.style;
 	GOGeometryOBR zero_obr;
 	GOGeometryOBR *obrs = NULL;
-	ArtVpath path[3];
+	GOGeometrySide label_anchor = GO_SIDE_AUTO;
+	GOPath *path = NULL;
 	double line_width;
 	double axis_length, axis_angle, label_padding;
 	double major_tick_len, minor_tick_len, tick_len;
@@ -1052,6 +1070,7 @@ axis_line_render (GogAxisBase *axis_base,
 	double cos_alpha, sin_alpha;
 	double pos, pos_x, pos_y;
 	double padding = gog_axis_base_get_padding (axis_base);
+	double label_size_max = 0;
 	unsigned i, tick_nbr, nobr = 0, *indexmap = NULL;
 	gboolean draw_major, draw_minor;
 	gboolean is_line_visible;
@@ -1059,31 +1078,24 @@ axis_line_render (GogAxisBase *axis_base,
 	go_geometry_cartesian_to_polar (w, h, &axis_length, &axis_angle);
 	cos_alpha = side == GO_SIDE_LEFT ? - sin (axis_angle) : + sin (axis_angle);
 	sin_alpha = side == GO_SIDE_LEFT ? + cos (axis_angle) : - cos (axis_angle);
-	
+
 	x -= gog_renderer_pt2r_x (renderer, padding * cos_alpha);
 	y -= gog_renderer_pt2r_y (renderer, padding * sin_alpha);
-	
+
 	axis_base_view->x_start = x;
 	axis_base_view->y_start = y;
 	axis_base_view->x_stop = x + w;
-	axis_base_view->y_stop = y + h;	
+	axis_base_view->y_stop = y + h;
 
 	is_line_visible = gog_style_is_line_visible (style);
 	line_width = gog_renderer_line_size (renderer, style->line.width) / 2;
+
 	if (is_line_visible)
 	{
-		path[0].code = ART_MOVETO;
-		path[1].code = ART_LINETO;
-		path[2].code = ART_END;
-
-		path[0].x = x;
-		path[0].y = y;
-		path[1].x = path[0].x + w;
-		path[1].y = path[0].y + h;
-		if (sharp)
-			gog_renderer_draw_sharp_path (renderer, path);
-		else
-			gog_renderer_draw_path (renderer, path);
+		path = go_path_new ();
+		go_path_set_options (path, sharp ? GO_PATH_OPTIONS_SHARP : 0);
+		go_path_move_to (path, x, y);
+		go_path_line_to (path, x + w, y + h);
 	}
 
 	map = gog_axis_map_new (axis_base->axis, 0., axis_length);
@@ -1112,6 +1124,17 @@ axis_line_render (GogAxisBase *axis_base,
 	if (draw_labels) {
 		obrs = g_new0 (GOGeometryOBR, tick_nbr);
 		indexmap = g_new0 (unsigned int, tick_nbr);
+		for (i = 0; i < tick_nbr; i++) {
+			if (ticks[i].label != NULL) {
+				GOGeometryOBR *obr = obrs + i;
+				gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, obr);
+				if (obr->w > label_size_max
+				    || obr->h > label_size_max) {
+					label_size_max = MAX (obr->w, obr->h);
+					label_anchor = go_geometry_calc_label_anchor (obr, axis_angle);
+				}
+			}
+		}
 	}
 
 	for (i = 0; i < tick_nbr; i++) {
@@ -1126,27 +1149,19 @@ axis_line_render (GogAxisBase *axis_base,
 			switch (ticks[i].type) {
 				case GOG_AXIS_TICK_MAJOR:
 					if (draw_major) {
-						path[0].x = major_out_x + pos_x;
-						path[1].x = major_in_x + pos_x;
-						path[0].y = major_out_y + pos_y;
-						path[1].y = major_in_y + pos_y;
-						if (sharp)
-							gog_renderer_draw_sharp_path (renderer, path);
-						else
-							gog_renderer_draw_path (renderer, path);
+						go_path_move_to (path, major_out_x + pos_x,
+								 major_out_y + pos_y);
+						go_path_line_to (path, major_in_x + pos_x,
+								 major_in_y + pos_y);
 					}
 					break;
 
 				case GOG_AXIS_TICK_MINOR:
 					if (draw_minor) {
-						path[0].x = minor_out_x + pos_x;
-						path[1].x = minor_in_x + pos_x;
-						path[0].y = minor_out_y + pos_y;
-						path[1].y = minor_in_y + pos_y;
-						if (sharp)
-							gog_renderer_draw_sharp_path (renderer, path);
-						else
-							gog_renderer_draw_path (renderer, path);
+						go_path_move_to (path, minor_out_x + pos_x,
+								 minor_out_y + pos_y);
+						go_path_line_to (path, minor_in_x + pos_x,
+								 minor_in_y + pos_y);
 					}
 					break;
 
@@ -1156,11 +1171,11 @@ axis_line_render (GogAxisBase *axis_base,
 		}
 
 		if (ticks[i].label != NULL && draw_labels) {
-			GOGeometryOBR *obr = obrs + nobr;
+			GOGeometryOBR *obr = obrs + i;
 			pos = gog_axis_map_to_view (map, ticks[i].position);
-			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, obr);
 			obr->w += label_padding;
-			go_geometry_calc_label_position (obr, axis_angle, tick_len, side);
+			go_geometry_calc_label_position (obr, axis_angle, tick_len,
+							 side, label_anchor);
 			obr->x += x + pos * cos (axis_angle);
 			obr->y += y + pos * sin (axis_angle);
 
@@ -1169,6 +1184,11 @@ axis_line_render (GogAxisBase *axis_base,
 		}
 	}
 
+	if (is_line_visible) {
+		gog_renderer_stroke_shape (renderer, path);
+		go_path_free (path);
+	}
+
 	/*
 	 * This is far from perfect, but at least things are regular.
 	 * The axis really needs to be queried to figure this out,
@@ -1178,9 +1198,12 @@ axis_line_render (GogAxisBase *axis_base,
 		int skip = 1;
 
 		for (i = skip; i < nobr; i += skip) {
-			gboolean overlap =
-				go_geometry_test_OBR_overlap (obrs + i,
-							      obrs + i + skip);
+			unsigned j;
+			gboolean overlap;
+
+			j = indexmap[i];
+			overlap = go_geometry_test_OBR_overlap (obrs + j,
+								obrs + indexmap[i - skip]);
 			if (overlap) {
 				skip++;
 				i = 0;
@@ -1191,8 +1214,8 @@ axis_line_render (GogAxisBase *axis_base,
 		for (i = 0; i < nobr; i += skip) {
 			unsigned j = indexmap[i];
 			GogViewAllocation label_pos;
-			label_pos.x = obrs[i].x;
-			label_pos.y = obrs[i].y;
+			label_pos.x = obrs[j].x;
+			label_pos.y = obrs[j].y;
 			gog_renderer_draw_text (renderer, ticks[j].label,
 						&label_pos, GTK_ANCHOR_CENTER,
 						TRUE);
@@ -1267,7 +1290,8 @@ axis_circle_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 		if (ticks[i].label != NULL && draw_labels) {
 			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, &txt_obr);
 			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, GO_SIDE_LEFT);
+			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, 
+							 GO_SIDE_LEFT, GO_SIDE_AUTO);
 			txt_obr.x += x;
 			txt_obr.y += y;
 			go_geometry_OBR_to_AABR (&txt_obr, &txt_aabr);
@@ -1294,10 +1318,11 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 	GogChartMapPolarData *parms = gog_chart_map_get_polar_parms (c_map);
 	GOGeometryOBR txt_obr, txt_obr_old = {0., 0., 0., 0., 0.};
 	GOGeometryOBR txt_obr_first;
-	ArtVpath *cpath, path[3];
+	GOPath *path;
 	double angle, offset, position, label_padding;
 	double start, stop;
 	double major_tick_len, minor_tick_len, tick_len;
+	double x0, y0, x1, y1;
 	unsigned i, step_nbr, tick_nbr;
 	gboolean draw_major, draw_minor;
 	gboolean is_line_visible;
@@ -1307,34 +1332,26 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 	gog_axis_map_get_extents (map, &offset , &position);
 	map = gog_chart_map_get_axis_map (c_map, 0);
 
+	path = go_path_new ();
+
 	if (is_discrete) {
 		gog_axis_map_get_extents (map, &start, &stop);
 		step_nbr = go_rint (parms->th1 - parms->th0) + 1;
-		cpath = art_new (ArtVpath, step_nbr + 2);
 		for (i = 0; i <= step_nbr; i++) {
-			gog_chart_map_2D_to_view (c_map, i + parms->th0, 
-						  position, &cpath[i].x, &cpath[i].y);
-			cpath[i].code = ART_LINETO;
+			gog_chart_map_2D_to_view (c_map, i + parms->th0, position, &x0, &y0);
+			if (i == 0)
+				go_path_move_to (path, x0, y0);
+			else
+				go_path_line_to (path, x0, y0);
 		}
-		cpath[0].code = ART_MOVETO;
-		cpath[step_nbr + 1].code = ART_END;
-		gog_renderer_draw_path (renderer, cpath);
-		g_free (cpath);
-	} else {
-		gog_renderer_draw_arc (renderer, parms->cx, parms->cy, parms->rx, parms->ry,
-				      -parms->th1, -parms->th0);
-	}
+	} else
+		go_path_arc (path, parms->cx, parms->cy, parms->rx, parms->ry,
+			     -parms->th1, -parms->th0);
 
 	is_line_visible = gog_style_is_line_visible (axis_base->base.style);
 	draw_major = axis_base->major.tick_in || axis_base->major.tick_out;
 	draw_minor = axis_base->minor.tick_in || axis_base->minor.tick_out;
 
-	if (is_line_visible) {
-		path[0].code = ART_MOVETO;
-		path[1].code = ART_LINETO;
-		path[2].code = ART_END;
-	}
-
 	minor_tick_len = gog_renderer_pt2r (renderer, axis_base->minor.size_pts);
 	major_tick_len = gog_renderer_pt2r (renderer, axis_base->major.size_pts);
 	tick_len = axis_base->major.tick_out ? major_tick_len :
@@ -1350,38 +1367,40 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 				case GOG_AXIS_TICK_MAJOR:
 					if (draw_major) {
 						gog_chart_map_2D_to_view (c_map, ticks[i].position, position,
-									  &path[0].x, &path[0].y);
+									  &x0, &y0);
 						if (axis_base->major.tick_in) {
-							path[1].x = path[0].x - major_tick_len * cos (angle);
-							path[1].y = path[0].y - major_tick_len * sin (angle);
+							x1 = x0 - major_tick_len * cos (angle);
+							y1 = y0 - major_tick_len * sin (angle);
 						} else {
-							path[1].x = path[0].x;
-							path[1].y = path[0].y;
+							x1 = x0;
+							y1 = y0;
 						}
 						if (axis_base->major.tick_out) {
-							path[0].x += major_tick_len * cos (angle);
-							path[0].y += major_tick_len * sin (angle);
+							x0 += major_tick_len * cos (angle);
+							y0 += major_tick_len * sin (angle);
 						}
-						gog_renderer_draw_path (renderer, path);
+						go_path_move_to (path, x0, y0);
+						go_path_line_to (path, x1, y1);
 					}
 					break;
 
 				case GOG_AXIS_TICK_MINOR:
 					if (draw_minor) {
 						gog_chart_map_2D_to_view (c_map, ticks[i].position, position,
-									  &path[0].x, &path[0].y);
+									  &x0, &y0);
 						if (axis_base->minor.tick_in) {
-							path[1].x = path[0].x - minor_tick_len * cos (angle);
-							path[1].y = path[0].y - minor_tick_len * sin (angle);
+							x1 = x0 - minor_tick_len * cos (angle);
+							y1 = y0 - minor_tick_len * sin (angle);
 						} else {
-							path[1].x = path[0].x;
-							path[1].y = path[0].y;
+							x1 = x0;
+							y1 = y0;
 						}
 						if (axis_base->minor.tick_out) {
-							path[0].x += minor_tick_len * cos (angle);
-							path[0].y += minor_tick_len * sin (angle);
+							x0 += minor_tick_len * cos (angle);
+							y0 += minor_tick_len * sin (angle);
 						}
-						gog_renderer_draw_path (renderer, path);
+						go_path_move_to (path, x0, y0);
+						go_path_line_to (path, x1, y1);
 					}
 					break;
 
@@ -1395,12 +1414,13 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 						  &label_pos.x, &label_pos.y);
 			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, &txt_obr);
 			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, GO_SIDE_LEFT);
+			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, 
+							 GO_SIDE_LEFT, GO_SIDE_AUTO);
 			label_pos.x += txt_obr.x;
 			label_pos.y += txt_obr.y;
 			txt_obr.x = label_pos.x;
 			txt_obr.y = label_pos.y;
-			if (!first_label_done || 
+			if (!first_label_done ||
 			    (!go_geometry_test_OBR_overlap (&txt_obr, &txt_obr_old) &&
 			     !go_geometry_test_OBR_overlap (&txt_obr, &txt_obr_first))) {
 				gog_renderer_draw_text (renderer, ticks[i].label,
@@ -1413,6 +1433,9 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 			}
 		}
 	}
+
+	gog_renderer_stroke_shape (renderer, path);
+	go_path_free (path);
 }
 
 static gboolean
