@@ -22,6 +22,7 @@
 #include <goffice/goffice-config.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-data-allocator.h>
 #include <goffice/graph/gog-grid-line.h>
 #include <goffice/graph/gog-renderer.h>
@@ -123,8 +124,11 @@ gog_grid_line_class_init (GogGridLineClass *klass)
 	style_klass->init_style = gog_grid_line_init_style;
 	
 	g_object_class_install_property (gobject_klass, GRID_LINE_PROP_IS_MINOR,
-		g_param_spec_boolean ("is-minor", "is-minor",
-			"Are these minor grid lines", FALSE, G_PARAM_READWRITE));
+		g_param_spec_boolean ("is-minor", 
+			_("Is-minor"),
+			_("Are these minor grid lines"), 
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 }
 
 GSF_CLASS (GogGridLine, gog_grid_line,
@@ -139,30 +143,18 @@ typedef GogViewClass	GogGridLineViewClass;
 #define GOG_GRID_LINE_VIEW(o)		(G_TYPE_CHECK_INSTANCE_CAST ((o), GOG_GRID_LINE_VIEW_TYPE, GogGridLineView))
 #define IS_GOG_GRID_LINE_VIEW(o)	(G_TYPE_CHECK_INSTANCE_TYPE ((o), GOG_GRID_LINE_VIEW_TYPE))
 
-static void
-fill_rectangle (GogRenderer *renderer, GogViewAllocation const *rectangle)
-{
-	ArtVpath *path = gog_renderer_get_rectangle_vpath (rectangle);
-	gog_renderer_draw_polygon (renderer, path, FALSE);
-	g_free (path);
-}
-
 static void
 gog_grid_line_xy_render (GogGridLine *grid_line, GogView *view,
-			 GogAxis *axis, GogAxisTick *ticks, unsigned int tick_nbr, 
+			 GogAxis *axis, GogAxisTick *ticks, unsigned int tick_nbr,
 			 GogViewAllocation const *plot_area,
 			 gboolean stripes)
 {
 	GogAxisMap *map = NULL;
 	GogAxisType axis_type = gog_axis_get_atype (axis);
-	GogViewAllocation rect;
-	ArtVpath path[3];
+	GogViewAllocation rect = {.0, .0, .0, .0};
+	GOPath *path;
 	unsigned int i;
 	gboolean stripe_started = FALSE;
-	
-	path[0].code = ART_MOVETO;
-	path[1].code = ART_LINETO;
-	path[2].code = ART_END;
 
 	switch (axis_type) {
 		case GOG_AXIS_X:
@@ -175,18 +167,21 @@ gog_grid_line_xy_render (GogGridLine *grid_line, GogView *view,
 			return;
 	}
 
+	path = go_path_new ();
+	go_path_set_options (path, GO_PATH_OPTIONS_SHARP);
+
 	switch (axis_type) {
-		case GOG_AXIS_X: 
+		case GOG_AXIS_X:
 			if (stripes) {
 				rect.y = plot_area->y;
 				rect.h = plot_area->h;
-				for (i = 0; i < tick_nbr; i++) 
+				for (i = 0; i < tick_nbr; i++)
 					if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 					    grid_line->is_minor) {
 						if (stripe_started) {
 							rect.w = gog_axis_map_to_view (map, ticks[i].position) -
 								rect.x;
-							fill_rectangle (view->renderer, &rect);
+							go_path_rectangle (path, rect.x, rect.y, rect.w, rect.h);
 							stripe_started = FALSE;
 						} else {
 							rect.x = gog_axis_map_to_view (map, ticks[i].position);
@@ -195,31 +190,33 @@ gog_grid_line_xy_render (GogGridLine *grid_line, GogView *view,
 					}
 				if (stripe_started) {
 					rect.w = (plot_area->x + plot_area->w) - rect.x;
-					fill_rectangle (view->renderer, &rect);
+					go_path_rectangle (path, rect.x, rect.y, rect.w, rect.h);
 				}
+				gog_renderer_fill_shape (view->renderer, path);
 			} else {
-				for (i = 0; i < tick_nbr; i++) 
+				double x;
+
+				for (i = 0; i < tick_nbr; i++)
 					if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 					    (ticks[i].type == GOG_AXIS_TICK_MINOR && grid_line->is_minor)) {
-						path[0].y = plot_area->y;
-						path[1].y = plot_area->y + plot_area->h;
-						path[0].x = gog_axis_map_to_view (map, ticks[i].position);
-						path[1].x = path[0].x;
-						gog_renderer_draw_sharp_path (view->renderer, path);
+						x = gog_axis_map_to_view (map, ticks[i].position);
+						go_path_move_to (path, x, plot_area->y);
+						go_path_line_to (path, x, plot_area->y + plot_area->h);
 					}
+				gog_renderer_stroke_shape (view->renderer, path);
 			}
 			break;
-		case GOG_AXIS_Y: 
+		case GOG_AXIS_Y:
 			if (stripes) {
 				rect.x = plot_area->x;
 				rect.w = plot_area->w;
-				for (i = 0; i < tick_nbr; i++) 
+				for (i = 0; i < tick_nbr; i++)
 					if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 					    grid_line->is_minor) {
 						if (stripe_started) {
 							rect.h = gog_axis_map_to_view (map, ticks[i].position) -
 								rect.y;
-							fill_rectangle (view->renderer, &rect);
+							go_path_rectangle (path, rect.x, rect.y, rect.w, rect.h);
 							stripe_started = FALSE;
 						} else {
 							rect.y = gog_axis_map_to_view (map, ticks[i].position);
@@ -228,29 +225,33 @@ gog_grid_line_xy_render (GogGridLine *grid_line, GogView *view,
 					}
 				if (stripe_started) {
 					rect.h = plot_area->y - rect.y;
-					fill_rectangle (view->renderer, &rect);
+					go_path_rectangle (path, rect.x, rect.y, rect.w, rect.h);
 				}
+				gog_renderer_fill_shape (view->renderer, path);
 			} else {
+				double y;
+
 				for (i = 0; i < tick_nbr; i++) {
 					if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 					    (ticks[i].type == GOG_AXIS_TICK_MINOR && grid_line->is_minor)) {
-						path[0].x = plot_area->x;
-						path[1].x = plot_area->x + plot_area->w;
-						path[0].y = gog_axis_map_to_view (map, ticks[i].position);
-						path[1].y = path[0].y; 
-						gog_renderer_draw_sharp_path (view->renderer, path);
+						y = gog_axis_map_to_view (map, ticks[i].position);
+						go_path_move_to (path, plot_area->x, y);
+						go_path_line_to (path, plot_area->x + plot_area->w, y);
 					}
 				}
+				gog_renderer_stroke_shape (view->renderer, path);
 			}
 			break;
 		default:
 			break;
 	}
+
+	go_path_free (path);
 	gog_axis_map_free (map);
 }
 
 static void
-gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view, 
+gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 			     GogAxis *axis, GogAxisTick *ticks, unsigned int tick_nbr,
 			     GogChart *chart,
 			     GogViewAllocation const *plot_area,
@@ -261,7 +262,8 @@ gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 	GogChartMap *c_map;
 	GogChartMapPolarData *parms;
 	GSList *axis_list;
-	ArtVpath *c_path;
+	GOPath *path;
+	double x, y;
 	unsigned int i, j;
 	gboolean stripe_started = FALSE;
 
@@ -271,9 +273,10 @@ gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 	circular_axis = GOG_AXIS (axis_list->data);
 	g_slist_free (axis_list);
 
-	c_map = gog_chart_map_new (GOG_CHART (chart), plot_area, 
+	c_map = gog_chart_map_new (GOG_CHART (chart), plot_area,
 				   circular_axis, axis, NULL, FALSE);
 	parms = gog_chart_map_get_polar_parms (c_map);
+	path = go_path_new ();
 
 	if (gog_axis_is_discrete (circular_axis)) {
 		unsigned step_nbr;
@@ -284,31 +287,30 @@ gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 		gog_axis_map_get_extents (map, &start, &stop);
 		step_nbr = go_rint (parms->th1 - parms->th0) + 1;
 		if (stripes) {
-			c_path = art_new (ArtVpath, 2 * step_nbr + 3);
 			for (i = 0; i < tick_nbr; i++) {
 				if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 				    grid_line->is_minor) {
 					if (stripe_started) {
 						for (j = 0; j <= step_nbr; j++) {
-							gog_chart_map_2D_to_view (c_map, 
-										  j + parms->th0 , 
-										  ticks[i].position, 
-										  &c_path[-j + 2 * step_nbr + 1].x,
-										  &c_path[-j + 2 * step_nbr + 1].y);
-							c_path[-j + 2 * step_nbr + 1].code = ART_LINETO;
+							gog_chart_map_2D_to_view (c_map,
+										  step_nbr - j + parms->th0 ,
+										  ticks[i].position,
+										  &x, &y);
+							go_path_line_to (path, x, y);
 						}
-						c_path[2 * step_nbr + 2].code = ART_END;
-						gog_renderer_draw_polygon (view->renderer, c_path, TRUE);
+						go_path_close (path);
 						stripe_started = FALSE;
 					} else {
 						for (j = 0; j <= step_nbr; j++) {
-							gog_chart_map_2D_to_view (c_map, 
-										  j + parms->th0 , 
-										  ticks[i].position, 
-										  &c_path[j].x, &c_path[j].y);
-							c_path[j].code = ART_LINETO;
+							gog_chart_map_2D_to_view (c_map,
+										  j + parms->th0 ,
+										  ticks[i].position,
+										  &x, &y);
+							if (j == 0)
+								go_path_move_to (path, x, y);
+							else
+								go_path_line_to (path, x, y);
 						}
-						c_path[0].code = ART_MOVETO;
 						stripe_started = TRUE;
 					}
 				}
@@ -317,35 +319,31 @@ gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 				map = gog_chart_map_get_axis_map (c_map, 1);
 				gog_axis_map_get_bounds (map, &minimum, &maximum);
 				for (j = 0; j <= step_nbr; j++) {
-					gog_chart_map_2D_to_view (c_map, 
-								  j + parms->th0 , 
-								  maximum, 
-								  &c_path[-j + 2 * step_nbr + 1].x,
-								  &c_path[-j + 2 * step_nbr + 1].y);
-					c_path[-j + 2 * step_nbr + 1].code = ART_LINETO;
+					gog_chart_map_2D_to_view (c_map, step_nbr - j + parms->th0 ,
+								  maximum, &x, &y);
+					go_path_line_to (path, x, y);
 				}
-				c_path[2 * step_nbr + 2].code = ART_END;
-				gog_renderer_draw_polygon (view->renderer, c_path, TRUE);
+				go_path_close (path);
 			}
+			gog_renderer_fill_shape (view->renderer, path);
 		} else {
-			c_path = art_new (ArtVpath, step_nbr + 2);
 			for (i = 0; i < tick_nbr; i++) {
 				if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 				    (ticks[i].type == GOG_AXIS_TICK_MINOR && grid_line->is_minor)) {
 					for (j = 0; j <= step_nbr; j++) {
-						gog_chart_map_2D_to_view (c_map, 
-									  j + parms->th0 , 
-									  ticks[i].position, 
-									  &c_path[j].x, &c_path[j].y);
-						c_path[j].code = ART_LINETO;
+						gog_chart_map_2D_to_view (c_map,
+									  j + parms->th0 ,
+									  ticks[i].position,
+									  &x, &y);
+						if (j == 0)
+							go_path_move_to (path, x, y);
+						else
+							go_path_line_to (path, x, y);
 					}
-					c_path[0].code = ART_MOVETO;
-					c_path[step_nbr + 1].code = ART_END;
-					gog_renderer_draw_path (view->renderer, c_path);
 				}
 			}
+			gog_renderer_stroke_shape (view->renderer, path);
 		}
-		g_free (c_path);
 	} else {
 		double position;
 		double previous_position = 0;
@@ -353,19 +351,18 @@ gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 
 		map = gog_chart_map_get_axis_map (c_map, 1);
 		if (stripes) {
-			for (i = 0; i < tick_nbr; i++) { 
+			for (i = 0; i < tick_nbr; i++) {
 				if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 				    grid_line->is_minor) {
 					position = gog_axis_map (map, ticks[i].position);
 					if (stripe_started) {
-						gog_renderer_draw_ring_wedge (view->renderer, 
-									      parms->cx, parms->cy,
-									      position * parms->rx, 
-									      position * parms->ry,
-									      previous_position * parms->rx, 
-									      previous_position * parms->ry,
-									      -parms->th1, -parms->th0,
-									      TRUE);
+						go_path_ring_wedge (path,
+								    parms->cx, parms->cy,
+								    position * parms->rx,
+								    position * parms->ry,
+								    previous_position * parms->rx,
+								    previous_position * parms->ry,
+								    -parms->th1, -parms->th0);
 						stripe_started = FALSE;
 					} else {
 						previous_position = position;
@@ -376,28 +373,30 @@ gog_grid_line_radial_render (GogGridLine *grid_line, GogView *view,
 			if (stripe_started) {
 				gog_axis_map_get_bounds (map, &minimum, &maximum);
 				position = gog_axis_map (map, maximum);
-				gog_renderer_draw_ring_wedge (view->renderer, 
-							      parms->cx, parms->cy,
-							      position * parms->rx, 
-							      position * parms->ry,
-							      previous_position * parms->rx, 
-							      previous_position * parms->ry,
-							      -parms->th1, -parms->th0,
-							      TRUE);
+				go_path_ring_wedge (path,
+						    parms->cx, parms->cy,
+						    position * parms->rx,
+						    position * parms->ry,
+						    previous_position * parms->rx,
+						    previous_position * parms->ry,
+						    -parms->th1, -parms->th0);
 			}
+			gog_renderer_fill_shape (view->renderer, path);
 		} else {
-			for (i = 0; i < tick_nbr; i++) { 
+			for (i = 0; i < tick_nbr; i++) {
 				if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 				    (ticks[i].type == GOG_AXIS_TICK_MINOR && grid_line->is_minor)) {
 					position = gog_axis_map (map, ticks[i].position);
-					gog_renderer_draw_arc (view->renderer, parms->cx, parms->cy,
-							       position * parms->rx, 
-							       position * parms->ry,
-							       -parms->th1, -parms->th0);
+					go_path_arc (path, parms->cx, parms->cy,
+						     position * parms->rx,
+						     position * parms->ry,
+						     -parms->th1, -parms->th0);
 				}
 			}
+			gog_renderer_stroke_shape (view->renderer, path);
 		}
 	}
+	go_path_free (path);
 	gog_chart_map_free (c_map);
 }
 
@@ -413,8 +412,9 @@ gog_grid_line_circular_render (GogGridLine *grid_line, GogView *view,
 	GogChartMap *c_map;
 	GogChartMapPolarData *parms;
 	GSList *axis_list;
-	ArtVpath path[3];
+	GOPath *path;
 	double start, stop;
+	double x0, y0, x1, y1;
 	unsigned int i;
 	gboolean stripe_started = FALSE;
 
@@ -425,12 +425,14 @@ gog_grid_line_circular_render (GogGridLine *grid_line, GogView *view,
 	radial_axis = GOG_AXIS (axis_list->data);
 	g_slist_free (axis_list);
 
-	c_map = gog_chart_map_new (GOG_CHART (chart), plot_area, 
+	c_map = gog_chart_map_new (GOG_CHART (chart), plot_area,
 				   axis, radial_axis, NULL, FALSE);
 	parms = gog_chart_map_get_polar_parms (c_map);
 	map = gog_chart_map_get_axis_map (c_map, 1);
 	gog_axis_map_get_extents (map, &start, &stop);
 
+	path = go_path_new ();
+
 	if (stripes) {
 		double last_theta = 0, theta, r_in, r_out;
 		double delta_theta = parms->th1 - parms->th0;
@@ -439,20 +441,19 @@ gog_grid_line_circular_render (GogGridLine *grid_line, GogView *view,
 		r_in = gog_axis_map (map, start);
 		r_out = gog_axis_map (map, stop);
 		map = gog_chart_map_get_axis_map (c_map, 0);
-		for (i = 0; i < tick_nbr; i++) 
+		for (i = 0; i < tick_nbr; i++)
 			if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 			    grid_line->is_minor) {
 				if (stripe_started) {
 					theta = gog_axis_map (map, ticks[i].position);
-					gog_renderer_draw_ring_wedge (view->renderer, 
-								      parms->cx, parms->cy,
-								      r_in * parms->rx, 
-								      r_in * parms->ry,
-								      r_out * parms->rx, 
-								      r_out * parms->ry,
-								      -parms->th0 - theta * delta_theta,
-								      -parms->th0 - last_theta * delta_theta,
-								      TRUE);
+					go_path_ring_wedge (path,
+							    parms->cx, parms->cy,
+							    r_in * parms->rx,
+							    r_in * parms->ry,
+							    r_out * parms->rx,
+							    r_out * parms->ry,
+							    -parms->th0 - theta * delta_theta,
+							    -parms->th0 - last_theta * delta_theta);
 					stripe_started = FALSE;
 				} else {
 					last_theta = gog_axis_map (map, ticks[i].position);
@@ -462,29 +463,29 @@ gog_grid_line_circular_render (GogGridLine *grid_line, GogView *view,
 		if (stripe_started) {
 			gog_axis_map_get_bounds (map, &minimum, &maximum);
 			theta = gog_axis_map (map, maximum);
-			gog_renderer_draw_ring_wedge (view->renderer, 
-						      parms->cx, parms->cy,
-						      r_in * parms->rx, 
-						      r_in * parms->ry,
-						      r_out * parms->rx, 
-						      r_out * parms->ry,
-						      -parms->th0 - theta * delta_theta,
-						      -parms->th0 - last_theta * delta_theta,
-						      TRUE);
+			go_path_ring_wedge (path,
+					    parms->cx, parms->cy,
+					    r_in * parms->rx,
+					    r_in * parms->ry,
+					    r_out * parms->rx,
+					    r_out * parms->ry,
+					    -parms->th0 - theta * delta_theta,
+					    -parms->th0 - last_theta * delta_theta);
 		}
+		gog_renderer_fill_shape (view->renderer, path);
 	} else {
-		gog_chart_map_2D_to_view (c_map, 0, start, &path[0].x, &path[0].y);
-		path[0].code = ART_MOVETO;
-		path[1].code = ART_LINETO;
-		path[2].code = ART_END;
-		for (i = 0; i < tick_nbr; i++) 
+		gog_chart_map_2D_to_view (c_map, 0, start, &x0, &y0);
+		for (i = 0; i < tick_nbr; i++)
 			if ((ticks[i].type == GOG_AXIS_TICK_MAJOR && !grid_line->is_minor) ||
 			    (ticks[i].type == GOG_AXIS_TICK_MINOR && grid_line->is_minor)) {
-				gog_chart_map_2D_to_view (c_map, ticks[i].position, stop, 
-							  &path[1].x, &path[1].y);
-				gog_renderer_draw_path (view->renderer, path);
+				gog_chart_map_2D_to_view (c_map, ticks[i].position, stop,
+							  &x1, &y1);
+				go_path_move_to (path, x0, y0);
+				go_path_line_to (path, x1, y1);
 			}
+		gog_renderer_stroke_shape (view->renderer, path);
 	}
+	go_path_free (path);
 	gog_chart_map_free (c_map);
 }
 
