@@ -650,10 +650,11 @@ cb_plot_family_menu_create (char const *id, GogPlotFamily *family,
 	if (g_hash_table_size (family->types) <= 0)
 		return;
 
-	axis_set = gog_chart_get_axis_set (GOG_CHART (closure->state->prop_object));
+	axis_set = gog_chart_get_axis_set (GOG_CHART (closure->state->prop_object)) 
+				& GOG_AXIS_SET_FUNDAMENTAL;
 
-	if (axis_set != GOG_AXIS_SET_UNKNOWN &&
-	    family->axis_set != axis_set)
+	if (axis_set != GOG_AXIS_SET_FUNDAMENTAL &&
+	    (family->axis_set & GOG_AXIS_SET_FUNDAMENTAL) != axis_set)
 		return;
 
 	menu = gtk_image_menu_item_new_with_label (_(family->name));
@@ -1428,22 +1429,34 @@ graph_guru_init (GraphGuruState *s)
 	s->button_navigate = graph_guru_init_button (s, "button_navigate");
 	s->button_ok	   = graph_guru_init_ok_button (s);
 
-/* FIXME FIXME FIXME move the docs down to libgoffice */
-	go_gtk_help_button_init	(glade_xml_get_widget (s->gui, "help_button"),
-				 go_sys_data_dir (), "gnumeric",
-				 "sect-graphics-plots");
-
 	return FALSE;
 }
 
 /**
- * dialog_graph_guru
- * @wb : The workbook to use as a parent window.
- * @graph : the graph to edit
- * @page : the page to start on.
+ * gog_guru_get_help_button :
+ * @guru : #GtkWidget  (the result of gog_guru).
  *
- * Pop up a graph guru.
- */
+ * Quick utility to allow application specific help.  Required until we clean
+ * up the relationship between goffice and gnumeric to decide which parts of
+ * the hlpe live where.
+ **/
+GtkWidget *
+gog_guru_get_help_button (GtkWidget *guru)
+{
+	GraphGuruState *state = g_object_get_data (G_OBJECT (guru), "state");
+	return state ? glade_xml_get_widget (state->gui, "help_button") : NULL;
+}
+
+/**
+ * gog_guru
+ * @graph	: the graph to edit
+ * @dalloc	: The data allocator to use for editing
+ * @cc		: Where to report errors
+ * @toplevel	: as a parent window.
+ * @closure	:
+ *
+ * Returns: and shows new graph guru.
+ **/
 GtkWidget *
 gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
 	  GOCmdContext *cc, GtkWindow *toplevel,
@@ -1498,6 +1511,8 @@ gog_guru (GogGraph *graph, GogDataAllocator *dalloc,
 	/* a candidate for merging into attach guru */
 	g_object_set_data_full (G_OBJECT (state->dialog),
 		"state", state, (GDestroyNotify) graph_guru_state_destroy);
+
+#warning for 0.5.0 remove this and have the callers set transient and handle show, then drop the toplevel arg
 	go_gtk_nonmodal_dialog (toplevel, GTK_WINDOW (state->dialog));
 	gtk_widget_show (GTK_WIDGET (state->dialog));
 
