@@ -34,15 +34,21 @@ typedef struct {
 } GogEditorPage;
 
 typedef struct {
-	unsigned	*store_page;	/* pointer to a place for storing last edited page */
-	GSList		*pages;		/* GogEditorPage */
+	unsigned	*store_page;		/* pointer to a place for storing last edited page */
+	GSList		*pages;			/* GogEditorPage */
+	GData		*registered_widgets;
 } GogEditor;
 
-GogEditor	*gog_editor_new 		 (void);
-void		 gog_editor_add_page 		 (GogEditor *editor, gpointer widget, char const *label);
-void 		 gog_editor_set_store_page  	 (GogEditor *editor, unsigned *store_page);
-gpointer 	 gog_editor_get_notebook 	 (GogEditor *editor);
-void 		 gog_editor_free 		 (GogEditor *editor);
+GogEditor	*gog_editor_new 		  (void);
+void 		 gog_editor_free 		  (GogEditor *editor);
+void		 gog_editor_add_page 		  (GogEditor *editor, gpointer widget, char const *label);
+#ifdef GOFFICE_WITH_GTK
+#include <gtk/gtkwidget.h>
+void 		 gog_editor_set_store_page  	  (GogEditor *editor, unsigned *store_page);
+void 		 gog_editor_register_widget 	  (GogEditor *editor, GtkWidget *widget);
+GtkWidget  	*gog_editor_get_registered_widget (GogEditor *editor, char const *name);
+GtkWidget 	*gog_editor_get_notebook 	  (GogEditor *editor);
+#endif
 
 typedef enum {
 	GOG_OBJECT_NAME_BY_ROLE	 = 1,
@@ -132,6 +138,7 @@ typedef struct {
 
 #define GOG_PARAM_PERSISTENT	(1 << (G_PARAM_USER_SHIFT+0))
 #define GOG_PARAM_FORCE_SAVE	(1 << (G_PARAM_USER_SHIFT+1))	/* even if the value == default */
+#define GOG_PARAM_POSITION	(1 << (G_PARAM_USER_SHIFT+2))	/* position parameters */
 
 GType gog_object_get_type (void);
 
@@ -162,10 +169,12 @@ void		  gog_object_can_reorder (GogObject const *obj,
 GogObject	 *gog_object_reorder	 (GogObject const *obj,
 					  gboolean inc, gboolean goto_max);
 
-GogObjectPosition gog_object_get_position_flags	   (GogObject const *obj, GogObjectPosition mask);
-gboolean          gog_object_set_position_flags    (GogObject *obj, GogObjectPosition flags, GogObjectPosition mask);
-void	          gog_object_get_manual_position   (GogObject *obj, GogViewAllocation *pos);
-void 		  gog_object_set_manual_position   (GogObject *obj, GogViewAllocation const *pos);
+GogObjectPosition gog_object_get_position_flags	       (GogObject const *obj, GogObjectPosition mask);
+gboolean          gog_object_set_position_flags        (GogObject *obj, GogObjectPosition flags, 
+							GogObjectPosition mask);
+gboolean 	  gog_object_is_default_position_flags (GogObject const *obj, char const *name);
+void	          gog_object_get_manual_position       (GogObject *obj, GogViewAllocation *pos);
+void 		  gog_object_set_manual_position       (GogObject *obj, GogViewAllocation const *pos);
 
 GogViewAllocation gog_object_get_manual_allocation (GogObject *gobj, 
 						    GogViewAllocation const *parent_allocation, 
