@@ -24,6 +24,7 @@
 #include <goffice/graph/goffice-graph.h>
 #include <goffice/utils/go-geometry.h>
 #include <goffice/utils/go-image.h>
+#include <goffice/utils/go-path.h>
 #include <glib-object.h>
 
 #include <gsf/gsf.h>
@@ -73,8 +74,9 @@ typedef enum
 G_BEGIN_DECLS
 
 /* We need to define an hair line width for the svg and gnome_print renderer. 
- * 0.24 pt is the dot size of a 300 dpi printer, if the plot is printed at scale 1:1 */
-#define GOG_RENDERER_HAIR_LINE_WIDTH	0.24
+ * 0.5 pt is approx. the dot size of a 150 dpi printer, if the plot is 
+ * printed at scale 1:1 */
+#define GOG_RENDERER_HAIRLINE_WIDTH_PTS	0.5
 
 #define GOG_RENDERER_GRIP_SIZE	4
 
@@ -83,57 +85,7 @@ G_BEGIN_DECLS
 #define IS_GOG_RENDERER(o)        (G_TYPE_CHECK_INSTANCE_TYPE((o), GOG_RENDERER_TYPE))
 #define GOG_RENDERER_GET_CLASS(o) (G_TYPE_INSTANCE_GET_CLASS((o), GOG_RENDERER_TYPE, GogRendererClass))
 
-GType gog_renderer_get_type            (void); 
-
-void  gog_renderer_request_update (GogRenderer *r);
-
-void  gog_renderer_push_style     	(GogRenderer *rend, GogStyle const *style);
-void  gog_renderer_push_selection_style (GogRenderer *renderer);
-void  gog_renderer_pop_style      	(GogRenderer *rend);
-
-void  gog_renderer_push_clip 	  (GogRenderer *rend, ArtVpath *clip_path);
-void  gog_renderer_pop_clip       (GogRenderer *rend);
-
-ArtVpath * gog_renderer_get_rectangle_vpath 	(GogViewAllocation const *rect);
-ArtBpath * gog_renderer_get_ring_wedge_bpath	(double cx, double cy, 
-						 double rx_out, double ry_out,
-						 double rx_in, double ry_in,
-						 double th0, double th1);
-
-void  gog_renderer_draw_sharp_path	(GogRenderer *rend, ArtVpath *path);
-void  gog_renderer_draw_sharp_polygon   (GogRenderer *rend, ArtVpath *path, gboolean narrow);
-void  gog_renderer_draw_sharp_rectangle (GogRenderer *r, GogViewAllocation const *rect);
-
-void  gog_renderer_draw_ring_wedge  	(GogRenderer *rend, double cx, double cy,
-					 double rx_out, double ry_out, 
-					 double rx_in, double ry_in,
-					 double th0, double th1,
-					 gboolean narrow);
-void  gog_renderer_draw_path      	(GogRenderer *rend, ArtVpath const *path);
-void  gog_renderer_draw_polygon   	(GogRenderer *rend, ArtVpath const *path, gboolean narrow);
-void  gog_renderer_draw_rectangle 	(GogRenderer *rend, GogViewAllocation const *rect);
-void  gog_renderer_draw_bezier_path     (GogRenderer *rend, ArtBpath const *path);
-
-void  gog_renderer_draw_text	  (GogRenderer *rend, char const *text,
-				   GogViewAllocation const *pos, GtkAnchorType anchor,
-				   gboolean use_markup);
-void  gog_renderer_draw_marker	  (GogRenderer *rend, double x, double y);
-void  gog_renderer_draw_grip	  (GogRenderer *renderer, double x, double y);
-
-#define gog_renderer_in_grip(x,y,grip_x,grip_y) ((x) >= ((grip_x) - (GOG_RENDERER_GRIP_SIZE)) && \
-						 (x) <= ((grip_x) + (GOG_RENDERER_GRIP_SIZE)) && \
-						 (y) >= ((grip_y) - (GOG_RENDERER_GRIP_SIZE)) && \
-						 (y) <= ((grip_y) + (GOG_RENDERER_GRIP_SIZE)))
-
-void  gog_renderer_get_text_OBR   (GogRenderer *rend, char const *text, 
-				   gboolean use_markup, GOGeometryOBR *obr);
-void  gog_renderer_get_text_AABR  (GogRenderer *rend, char const *text, 
-				   gboolean use_markup, GOGeometryAABR *aabr);
-
-#define gog_renderer_draw_arc(r,cx,cy,rx,ry,th0,th1) \
-	gog_renderer_draw_ring_wedge (r,cx,cy,rx,ry,-1.,-1.,th0,th1,FALSE)
-#define gog_renderer_draw_pie_wedge(r,cx,cy,rx,ry,th0,th1,narrow) \
-	gog_renderer_draw_ring_wedge (r,cx,cy,rx,ry,0.,0.,th0,th1,narrow)
+GType gog_renderer_get_type            (void);
 
 /* measurement */
 double gog_renderer_line_size	  	(GogRenderer const *r, double width);
@@ -141,15 +93,63 @@ double gog_renderer_pt2r_x   	  	(GogRenderer const *r, double d);
 double gog_renderer_pt2r_y   	  	(GogRenderer const *r, double d);
 double gog_renderer_pt2r   	  	(GogRenderer const *r, double d);
 
+double gog_renderer_get_hairline_width_pts	(GogRenderer const *r);
+
+void  gog_renderer_stroke_serie    	(GogRenderer *renderer, GOPath const *path);
+void  gog_renderer_fill_serie		(GogRenderer *renderer, GOPath const *path, GOPath const *close_path);
+
+void  gog_renderer_draw_shape 		(GogRenderer *renderer, GOPath const *path);
+void  gog_renderer_stroke_shape		(GogRenderer *renderer, GOPath const *path);
+void  gog_renderer_fill_shape 		(GogRenderer *renderer, GOPath const *path);
+
+void  gog_renderer_draw_rectangle	(GogRenderer *rend, GogViewAllocation const *rect);
+void  gog_renderer_stroke_rectangle 	(GogRenderer *rend, GogViewAllocation const *rect);
+
+void  gog_renderer_draw_grip	  		(GogRenderer *renderer, double x, double y);
+void  gog_renderer_draw_selection_rectangle	(GogRenderer *renderer, GogViewAllocation const *rectangle);
+
+#define gog_renderer_in_grip(x,y,grip_x,grip_y) ((x) >= ((grip_x) - (GOG_RENDERER_GRIP_SIZE)) && \
+						 (x) <= ((grip_x) + (GOG_RENDERER_GRIP_SIZE)) && \
+						 (y) >= ((grip_y) - (GOG_RENDERER_GRIP_SIZE)) && \
+						 (y) <= ((grip_y) + (GOG_RENDERER_GRIP_SIZE)))
+
+void  gog_renderer_draw_marker	  (GogRenderer *rend, double x, double y);
+
+void  gog_renderer_draw_text	  (GogRenderer *rend, char const *text,
+				   GogViewAllocation const *pos, GtkAnchorType anchor,
+				   gboolean use_markup);
+
+void  gog_renderer_get_text_OBR   (GogRenderer *rend, char const *text,
+				   gboolean use_markup, GOGeometryOBR *obr);
+void  gog_renderer_get_text_AABR  (GogRenderer *rend, char const *text,
+				   gboolean use_markup, GOGeometryAABR *aabr);
+
+void  gog_renderer_push_style     	(GogRenderer *rend, GogStyle const *style);
+void  gog_renderer_pop_style      	(GogRenderer *rend);
+
+void  gog_renderer_push_clip 	  	(GogRenderer *rend, GOPath const *path);
+void  gog_renderer_push_clip_rectangle 	(GogRenderer *rend, double x, double y, double w, double h);
+void  gog_renderer_pop_clip       	(GogRenderer *rend);
+
 /* utilities for cairo/libart transition */
 GogRenderer 	*gog_renderer_new_for_pixbuf 	(GogGraph *graph);
 gboolean	 gog_renderer_update		(GogRenderer *renderer, double w, double h, double zoom);
-GdkPixbuf 	*gog_renderer_get_pixbuf 	(GogRenderer *renderer); 
+GdkPixbuf 	*gog_renderer_get_pixbuf 	(GogRenderer *renderer);
 
 GogRenderer 	*gog_renderer_new_for_format 	(GogGraph *graph, GOImageFormat format);
 gboolean 	 gog_renderer_export_image 	(GogRenderer *renderer, GOImageFormat format,
 						 GsfOutput *output, double x_dpi, double y_dpi);
 
+void  gog_renderer_request_update (GogRenderer *r);
+
+/* Deprecated API */
+void  gog_renderer_draw_sharp_path	(GogRenderer *rend, ArtVpath *path);
+void  gog_renderer_draw_sharp_polygon   (GogRenderer *rend, ArtVpath *path, gboolean narrow);
+
+void  gog_renderer_draw_path      	(GogRenderer *rend, ArtVpath const *path);
+void  gog_renderer_draw_polygon   	(GogRenderer *rend, ArtVpath const *path, gboolean narrow);
+void  gog_renderer_draw_bezier_path     (GogRenderer *rend, ArtBpath const *path);
+
 G_END_DECLS
 
 #endif /* GOG_RENDERER_H */
