@@ -24,7 +24,7 @@
 #include "gog-1.5d.h"
 #include <goffice/graph/gog-series-lines.h>
 #include <goffice/graph/gog-view.h>
-#include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-style.h>
 #include <goffice/graph/gog-axis.h>
@@ -569,9 +569,9 @@ gog_line_view_render (GogView *view, GogViewAllocation const *bbox)
 
 		}
 	}
-	
-	gog_renderer_push_clip (view->renderer, 
-				gog_renderer_get_rectangle_vpath (&view->allocation));
+
+	gog_renderer_push_clip_rectangle (view->renderer, view->allocation.x, view->allocation.y,
+					  view->allocation.w, view->allocation.h);
 
 	for (i = 0; i < num_series; i++) {
 
