@@ -24,6 +24,7 @@
 #include <goffice/data/go-data.h>
 #include <goffice/graph/gog-axis.h>
 #include <goffice/graph/gog-chart.h>
+#include <goffice/graph/gog-chart-map.h>
 #include <goffice/graph/gog-renderer.h>
 #include <goffice/graph/gog-series-lines.h>
 #include <goffice/utils/go-format.h>
@@ -82,10 +83,7 @@ gog_histogram_plot_update (GogObject *obj)
 	if (!gog_series_is_valid (GOG_SERIES (series)) || series->base.num_elements == 0)
 			return;
 	
-	if (series->x) {
-		g_free (series->x);
-		series->x = NULL;
-	}
+	g_free (series->x);
 	series->x = g_new (double, series->base.num_elements);
 	if (series->base.values[0].data != NULL) {
 		x_vals = go_data_vector_get_values (GO_DATA_VECTOR (series->base.values[0].data));
@@ -107,10 +105,8 @@ gog_histogram_plot_update (GogObject *obj)
 		model->x.maxima = x_max;
 		gog_axis_bound_changed (model->base.axis[0], GOG_OBJECT (model));
 	}
-	if (series->y) {
-		g_free (series->y);
-		series->y = NULL;
-	}
+	g_free (series->y);
+	series->y = NULL;
 	if (series->base.values[1].data != NULL) {
 		if (x_vals) {
 			series->y = g_new (double, series->base.num_elements);
@@ -218,13 +214,18 @@ gog_histogram_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogChartMap *chart_map;
 	GogAxisMap *x_map, *y_map;
 	GogViewAllocation const *area;
-	GogHistogramPlotSeries const *series = GOG_HISTOGRAM_PLOT_SERIES (model->base.series->data);
+	GogHistogramPlotSeries const *series;
 	double *x_vals = NULL, *y_vals, curx, cury;
-	unsigned i, j, nb = (series->base.num_elements + 1) * 2 + 2;
+	unsigned i, j, nb;
 	ArtVpath *path ;
 	GSList *ptr;
-	GogStyle *style = GOG_STYLED_OBJECT (series)->style;
+	GogStyle *style;
 
+	if (model->base.series == NULL)
+		return;
+	series = GOG_HISTOGRAM_PLOT_SERIES (model->base.series->data);
+	nb = (series->base.num_elements + 1) * 2 + 2;
+	style = GOG_STYLED_OBJECT (series)->style;
 	if (series->base.num_elements < 1)
 		return;
 	area = gog_chart_view_get_plot_area (view->parent);
@@ -266,7 +267,7 @@ gog_histogram_plot_view_render (GogView *view, GogViewAllocation const *bbox)
 	path[j++].y = path[0].y;
 	path[j].code = ART_END;
 	gog_renderer_push_style (view->renderer, style);
-	gog_renderer_draw_polygon (view->renderer, path, FALSE);
+	gog_renderer_draw_sharp_polygon (view->renderer, path, FALSE);
 
 	if (series->droplines) {
 		ArtVpath droppath[3];
@@ -464,14 +465,10 @@ gog_histogram_plot_series_finalize (GObject *obj)
 {
 	GogHistogramPlotSeries *series = GOG_HISTOGRAM_PLOT_SERIES (obj);
 
-	if (series->y != NULL) {
-		g_free (series->y); 
-		series->y = NULL;
-	}
-	if (series->x != NULL) {
-		g_free (series->x); 
-		series->x = NULL;
-	}
+	g_free (series->y);
+	series->y = NULL;
+	g_free (series->x);
+	series->x = NULL;
 
 	G_OBJECT_CLASS (series_parent_klass)->finalize (obj);
 }
