@@ -102,10 +102,11 @@ gog_pie_series_element_class_init (GogPieSeriesElementClass *klass)
 #endif
 
 	g_object_class_install_property (gobject_klass, ELEMENT_SEPARATION,
-		g_param_spec_float ("separation", "separation",
-			"Amount a slice is extended as a percentage of the radius",
+		g_param_spec_float ("separation", 
+			_("Separation"),
+			_("Amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 }
 
 GSF_DYNAMIC_CLASS (GogPieSeriesElement, gog_pie_series_element,
@@ -228,25 +229,29 @@ gog_pie_plot_class_init (GogPlotClass *plot_klass)
 	gog_klass->view_type	= gog_pie_view_get_type ();
 
 	g_object_class_install_property (gobject_klass, PLOT_PROP_INITIAL_ANGLE,
-		g_param_spec_float ("initial-angle", "initial-angle",
-			"Degrees clockwise from 12 O'Clock.",
+		g_param_spec_float ("initial-angle", 
+			_("Initial angle"),
+			_("Degrees clockwise from 12 O'Clock."),
 			0, G_MAXFLOAT, 0.,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_DEFAULT_SEPARATION,
-		g_param_spec_float ("default-separation", "default-separation",
-			"Default amount a slice is extended as a percentage of the radius",
+		g_param_spec_float ("default-separation", 
+			_("Default separation"),
+			_("Default amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_IN_3D,
-		g_param_spec_boolean ("in-3d", "in-3d",
-			"Draw 3d wedges",
+		g_param_spec_boolean ("in-3d", 
+			_("In 3d"),
+			_("Draw 3d wedges"),
 			FALSE,
-			G_PARAM_READWRITE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, PLOT_PROP_SPAN,
-		g_param_spec_float ("span", _("span"),
+		g_param_spec_float ("span", 
+			_("Span"),
 			_("Total angle used as a percentage of the full circle"),
 			10., 100., 100.,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	{
 		static GogSeriesDimDesc dimensions[] = {
@@ -362,10 +367,11 @@ gog_ring_plot_class_init (GogPiePlotClass *pie_plot_klass)
 	gog_klass->populate_editor = gog_ring_plot_populate_editor;
 
 	g_object_class_install_property (gobject_klass, RING_PLOT_PROP_CENTER_SIZE,
-		g_param_spec_float ("center-size", "center-size",
-			"Size of the center hole as a percentage of the radius",
+		g_param_spec_float ("center-size", 
+			_("Center-size"),
+			_("Size of the center hole as a percentage of the radius"),
 			0, 100., 0.,
-			G_PARAM_READWRITE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 
 	plot_klass->desc.num_series_min = 1;
 	plot_klass->desc.num_series_max = G_MAXINT;
@@ -452,9 +458,8 @@ static void
 gog_tool_move_pie_render (GogView *view)
 {
 	GogViewAllocation rectangle;
-	ArtVpath *path;
 	double r = view->allocation.h;
-	
+
 	if (r > view->allocation.w)
 		r = view->allocation.w;
 
@@ -463,9 +468,7 @@ gog_tool_move_pie_render (GogView *view)
 	rectangle.w = r;
 	rectangle.h = r;
 
-	path = gog_renderer_get_rectangle_vpath (&rectangle);
-	gog_renderer_draw_sharp_path (view->renderer, path);
-	art_free (path);
+	gog_renderer_draw_selection_rectangle (view->renderer, &rectangle);
 }
 
 static void
@@ -474,7 +477,7 @@ gog_tool_move_pie_init (GogToolAction *action)
 	GogPiePlot *pie = GOG_PIE_PLOT (action->view->model);
 	MovePieData *data = g_new0 (MovePieData, 1);
 	GogViewAllocation area = action->view->allocation;
-	
+
 	data->r = area.h;
 	if (data->r > area.w)
 		data->r = area.w;
@@ -550,15 +553,15 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 {
 	GogPiePlot const *model = GOG_PIE_PLOT (view->model);
 	GogPieSeries const *series = NULL;
-	double separated_cx, separated_cy, cx, cy, r, dt, tmp, theta, len, *vals, scale;
+	double separated_cx, separated_cy, cx, cy, r, theta, len, *vals, scale;
 	double default_sep;
-	unsigned elem, j, n, k;
-	ArtVpath path [MAX_ARC_SEGMENTS*2 + 4];
+	unsigned elem, k;
+	GOPath *path;
 	GogTheme *theme = gog_object_get_theme (GOG_OBJECT (model));
 	GogStyle *style;
 	GSList *ptr;
 	unsigned num_series = 0;
-	unsigned index, mirror = 0; /* init mirror because gcc is silly */
+	unsigned index;
 	double center_radius;
 	double center_size = 0.0;
 	double r_ext, r_int, r_tot;
@@ -571,7 +574,7 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	/* compute number of valid series */
 	for (ptr = model->base.series ; ptr != NULL ; ptr = ptr->next) {
 		series = ptr->data;
-	  	if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
+		if (!gog_series_is_valid (GOG_SERIES (ptr->data)))
 			continue;
 		num_series++;
 	}
@@ -584,11 +587,11 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 	if ((style = gog_styled_object_get_style (GOG_STYLED_OBJECT (series))))
 		outline_width_max = gog_renderer_line_size (view->renderer, style->outline.width);
 	for (overrides = gog_series_get_overrides (GOG_SERIES (series));
-	     overrides != NULL; 
+	     overrides != NULL;
 	     overrides = overrides->next) {
 		separation = GOG_PIE_SERIES_ELEMENT (overrides->data)->separation;
 		if (separation_max < separation)
-			separation_max = separation; 
+			separation_max = separation;
 		style = gog_styled_object_get_style (GOG_STYLED_OBJECT (overrides->data));
 		if (outline_width_max < style->outline.width)
 			outline_width_max = style->outline.width;
@@ -689,6 +692,8 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 		r_tot /= 2. * (1. + model->default_separation + separation_max);
 	}
 
+	path = go_path_new ();
+
 	default_sep = r_tot * model->default_separation;
 	center_radius = r_tot * center_size;
 	r = r_tot * (1. - center_size);
@@ -725,27 +730,27 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			len = fabs (vals[k]) * scale;
 			if (!go_finite (len) || len < 1e-3)
 				continue;
-			
+
 			gpse = NULL;
 			if ((overrides != NULL) &&
 			    (GOG_SERIES_ELEMENT (overrides->data)->index == k)) {
 				gpse = GOG_PIE_SERIES_ELEMENT (overrides->data);
 				overrides = overrides->next;
-				gog_renderer_push_style (view->renderer, 
+				gog_renderer_push_style (view->renderer,
 					gog_styled_object_get_style (
 						GOG_STYLED_OBJECT (gpse)));
 			} else if (model->base.vary_style_by_element)
 				gog_theme_fillin_style (theme, style, GOG_OBJECT (series),
 							model->base.index_num + k, FALSE);
-				
+
 			/* only separate the outer ring */
 			separated_cx = cx;
 			separated_cy = cy;
-			if (num_series == index && (default_sep > 0. || gpse != NULL)) { 
+			if (num_series == index && (default_sep > 0. || gpse != NULL)) {
 
 				separation = default_sep;
 
-				if (gpse != NULL) 
+				if (gpse != NULL)
 					separation += gpse->separation * r_tot;
 
 				separated_cx += separation * cos (theta + len/2.);
@@ -753,40 +758,14 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 			}
 			theta += len;
 
-			n = MAX_ARC_SEGMENTS * len / (2 * M_PI);
-			if (n < 6)
-				n = 6;
-			else if (n > MAX_ARC_SEGMENTS)
-				n = MAX_ARC_SEGMENTS;
-
-			dt = (double)-len / (double)n;
-			path[0].code = ART_MOVETO;
-			path[0].x = separated_cx;
-			path[0].y = separated_cy;
-			if (has_hole) {
-				path[0].x += r_int * cos (theta);
-				path[0].y += r_int * sin (theta);
-				mirror = 2*n + 3;
-				path[mirror].code = ART_END;
-			} else {
-				path[n+2].code = ART_LINETO;
-				path[n+2].x = separated_cx;
-				path[n+2].y = separated_cy;
-				path[n+3].code = ART_END;
-			}
-			for (tmp = theta, j = 0; j++ <= n ; tmp += dt) {
-				path[j].code = ART_LINETO;
-				path[j].x = separated_cx + r_ext * cos (tmp);
-				path[j].y = separated_cy + r_ext * sin (tmp);
-				if (has_hole) {
-					path[mirror - j].code = ART_LINETO;
-					path[mirror - j].x = separated_cx + r_int * cos (tmp);
-					path[mirror - j].y = separated_cy + r_int * sin (tmp);
-				}
-			}
-
-			gog_renderer_draw_polygon (view->renderer, path,
-						   r * len < 5 /* drop outline for thin segments */);
+			go_path_ring_wedge (path, separated_cx, separated_cy,
+					    r_ext, r_ext,
+					    has_hole ? r_int : 0.0,
+					    has_hole ? r_int : 0.0,
+					    theta - len,
+					    theta);
+			gog_renderer_draw_shape (view->renderer, path);
+			go_path_clear (path);
 
 			if (gpse != NULL)
 				gog_renderer_pop_style (view->renderer);
@@ -798,6 +777,8 @@ gog_pie_view_render (GogView *view, GogViewAllocation const *bbox)
 
 		index ++;
 	}
+
+	go_path_free (path);
 }
 
 static void
@@ -914,15 +895,17 @@ gog_pie_series_class_init (GObjectClass *gobject_klass)
 	gobject_klass->get_property = gog_pie_series_get_property;
 
 	g_object_class_install_property (gobject_klass, SERIES_PROP_INITIAL_ANGLE,
-		g_param_spec_float ("initial-angle", "initial-angle",
-			"Degrees clockwise from 12 O'Clock.",
+		g_param_spec_float ("initial-angle", 
+			_("Initial-angle"),
+			_("Degrees clockwise from 12 O'Clock"),
 			0, G_MAXFLOAT, 0.,
-			G_PARAM_READWRITE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_SEPARATION,
-		g_param_spec_float ("separation", "separation",
-			"Default amount a slice is extended as a percentage of the radius",
+		g_param_spec_float ("separation", 
+			_("Separation"),
+			_("Default amount a slice is extended as a percentage of the radius"),
 			0, G_MAXFLOAT, 0.,
-			G_PARAM_READWRITE));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE));
 }
 
 GSF_DYNAMIC_CLASS (GogPieSeries, gog_pie_series,
